<?php
/**
 * @package   admintoolswp
 * @copyright Copyright (c)2017-2019 Nicholas K. Dionysopoulos / Akeeba Ltd
 * @license   GNU GPL version 3 or later
 */

namespace Akeeba\AdminTools\Admin\Helper;

use Akeeba\AdminTools\Library\Registry\Registry;
use Akeeba\Engine\Factory;
use Akeeba\Engine\Platform;

defined('ADMINTOOLSINC') or die();

/**
 * A helper class to handle the storage of Plugin and Platform configuration values in the database
 */
class Params
{
	/** @var  Registry  The internal values registry */
	private $config = null;

	private $platformKeys = array('file_extensions', 'exclude_folders', 'exclude_files', 'min_exec_time', 'max_exec_time', 'run_time_bias');

	/** @var  Params  Singleton instance */
	static $instance = null;

	/**
	 * Singleton implementation
	 *
	 * @return \Akeeba\AdminTools\Admin\Helper\Params
	 *
	 *
	 */
	public static function &getInstance()
	{
		if (is_null(static::$instance))
		{
			static::$instance = new Params();
		}

		return static::$instance;
	}

	/**
	 * Storage constructor.
	 */
	public function __construct()
	{
		$this->load();
	}

	/**
	 * Retrieve a value
	 *
	 * @param   string  $key      The key to retrieve
	 * @param   mixed   $default  Default value if the key is not set
	 *
	 * @return  mixed  The key's value (or the default value)
	 */
	public function getValue($key, $default = null)
	{
		// Lazy load platform config
		if (in_array($key, $this->platformKeys))
		{
			$this->_loadPlatformConfig();
		}

		return $this->config->get($key, $default);
	}

	/**
	 * Set a configuration value
	 *
	 * @param   string  $key    Key to set
	 * @param   mixed   $value  Value to set the key to
	 * @param   bool    $save   Should I save everything to database?
	 *
	 * @return  mixed  The old value of the key
	 */
	public function setValue($key, $value, $save = false)
	{
		$x = $this->config->set($key, $value);

		if ($save)
		{
			$this->save();
		}

		return $x;
	}

	/**
	 * Resets the storage
	 *
	 * @param   bool  $save  Should I save everything to database?
	 */
	public function resetContents($save = false)
	{
		$this->config->loadArray(array());

		if ($save)
		{
			$this->save();
		}
	}

	/**
	 * Load the configuration information from the database
	 *
	 * @return  void
	 */
	public function load()
	{
		$data = '';

		$db = Wordpress::getDb();
		$query = $db->getQuery(true)
					->select($db->qn('at_value'))
			        ->from($db->qn('#__admintools_storage'))
			        ->where($db->qn('at_key').' = '.$db->q('pparams'));

		try
		{
			$data = $db->setQuery($query)->loadResult();
		}
		catch (\RuntimeException $e)
		{

		}

		$this->config = new Registry($data);
	}

	/**
	 * Loads platform configuration inside the current registry object.
	 * This is a separate function so we can lazy load the platform configuration and avoid "polluting" the global state.
	 * For simplicity sake, this class handles the params stored inside the database and inside the platform, however in
	 * 90% of cases we will only need the plugin params and not the platform ones.
	 */
	private function _loadPlatformConfig()
	{
		static $loaded = false;

		if ($loaded)
		{
			return;
		}

		// Now load the configuration stored inside Akeeba Engine
		if (!defined('AKEEBAENGINE'))
		{
			define('AKEEBAENGINE', 1);
		}

		require_once ADMINTOOLSWP_PATH . '/app/engine/Autoloader.php';

		// Load the platform
		Platform::addPlatform('filescan', ADMINTOOLSWP_PATH . '/app/platform/Filescan');

		// Load the engine configuration
		Platform::getInstance()->load_configuration(1);

		$aeconfig = Factory::getConfiguration();

		// These values are stored inside the profiles table, for consistency with Admin Tools
		$this->config->set('file_extensions', explode('|', $aeconfig->get('akeeba.basic.file_extensions', 'php|phps|php3|inc')));
		$this->config->set('exclude_folders', explode('|', $aeconfig->get('akeeba.basic.exclude_folders', '')));
		$this->config->set('exclude_files', explode('|', $aeconfig->get('akeeba.basic.exclude_files', '')));

		$this->config->set('min_exec_time', $aeconfig->get('akeeba.tuning.min_exec_time', 1000));
		$this->config->set('max_exec_time', $aeconfig->get('akeeba.tuning.max_exec_time', 5));
		$this->config->set('run_time_bias', $aeconfig->get('akeeba.tuning.run_time_bias', 75));

		$loaded = true;
	}

	/**
	 * Save the configuration information to the database
	 *
	 * @return  void
	 */
	public function save()
	{
		$platformBasicKeys  = array('file_extensions', 'exclude_folders', 'exclude_files');
		$platformTuningKeys = array('min_exec_time', 'max_exec_time', 'run_time_bias');

		$platformData = array();
		$data = $this->config->toArray();

		foreach ($data as $key => $value)
		{
			if (in_array($key, $platformBasicKeys) || in_array($key, $platformTuningKeys))
			{
				// Remove keys that should be stored inside the platform table
				$platformData[$key] = $value;
				unset($data[$key]);
			}
		}

		$data = json_encode($data);

		$db = Wordpress::getDb();
		$query = $db->getQuery(true)
					->delete($db->qn('#__admintools_storage'))
					->where($db->qn('at_key').' = '.$db->q('pparams'));

		$db->setQuery($query)->execute();

		$object = (object) array(
			'at_key'   => 'pparams',
			'at_value' => $data
		);

		$db->insertObject('#__admintools_storage', $object);

        // Now load the configuration stored inside Akeeba Engine
        if (!defined('AKEEBAENGINE'))
        {
            define('AKEEBAENGINE', 1);
        }

        require_once ADMINTOOLSWP_PATH . '/app/engine/Autoloader.php';

        Platform::addPlatform('filescan', ADMINTOOLSWP_PATH . '/app/platform/Filescan');

		// Then save platform data
		$platform = Platform::getInstance();
		$platform->load_configuration(1);

		$config = Factory::getConfiguration();

		// Remove the protection so we can update the values
		$protectedNodes = $config->getProtectedKeys();
		$config->resetProtectedKeys();

		foreach ($platformData as $key => $value)
		{
			// If I have CR or LF chars, let's convert them into an array
			if (strpos($value, "\r") !== false || strpos($value, "\n") !== false)
			{
				$value = str_replace("\r", "\n", $value);
				$value = str_replace("\n\n", "\n", $value);
				$value = explode("\n", $value);
			}

			// If I have an array, let's pack them together
			if (is_array($value))
			{
				$value = implode('|', $value);
			}

			// Calculate the right prefix for each value
			$prefix = 'akeeba.basic.';

			if (in_array($key, $platformTuningKeys))
			{
				$prefix = 'akeeba.tuning.';
			}

			$config->set($prefix.$key, $value);
		}

		// Finally re-apply the protection
		$config->setKeyProtection($protectedNodes, true);

		$platform->save_configuration(1);
	}
}
